﻿using LifeLogApp.Classes;
using LifeLogApp.ViewModel;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace LifeLogApp.Services;

public class UploadService
{
    Serilog.ILogger logger;
    GeoService geoService;
    MainViewModel mainViewModel;

    public Task task;
    public CancellationToken cancellationToken;

    public UploadService(Serilog.ILogger logger, GeoService geoService, MainViewModel mainViewModel)
    {
        this.logger = logger;
        this.geoService = geoService;
        this.mainViewModel = mainViewModel;

        logger.Verbose("{Method}: Start", AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

        task = Task.Run(() => Run(), cancellationToken);
    }

    public async void Run()
    {
        logger.Verbose("{Method}: Start", AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

        while (!cancellationToken.IsCancellationRequested)
        {
            var doUpload = mainViewModel.DoUpload;
            logger.Verbose(
                "{Method}: DoUpload={DoUpload}, HasApiKey={HasApiKey}",
                AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()),
                doUpload,
                !string.IsNullOrWhiteSpace(mainViewModel.ApiKey));
            if (doUpload && !string.IsNullOrWhiteSpace(mainViewModel.ApiKey))
            {
                logger.Verbose("{Method}: While", AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

                var currentLocation = await geoService.GetCurrentLocationAsync();

                var mediaToUpload = Directory.GetFiles(
                    AppHelper.AppPathEventsUpload,
                    "*.*",
                    SearchOption.AllDirectories);
                foreach (var m in mediaToUpload)
                {
                    logger.Verbose(
                        "{Method}: Start Upload Media {Media}",
                        AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()),
                        m);

                    var client = new RestClient(
                        $"https://my.unipoint.se/LifeLog_Dev/api/File?name={System.IO.Path.GetFileName(m)}");
                    client.Timeout = -1;
                    var request = new RestRequest(Method.POST);
                    request.AddHeader("ApiKey", mainViewModel.ApiKey);
                    request.AddFile("", m);

                    var fullUrl = client.BuildUri(request);
                    logger.Verbose("FullUrl={FullUrl}", fullUrl);

                    IRestResponse response = client.Execute(request);
                    logger.Verbose("Response={StatusCode}, {Content}", response.StatusCode, response.Content);

                    if (response.IsSuccessful)
                    {
                        logger.Information(
                            "{Method}: Media Uploaded OK {Media}",
                            AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()),
                            m);
                        try
                        {
                            File.Delete(m);
                        }
                        catch (Exception ex)
                        {
                            logger.Error(ex, "File.Delete");
                        }
                    }

                    mainViewModel.RaiseOnPropertyChangedUploadFiles();
                }
            }

            await Task.Delay(5000);
        }
    }
}
